#
# Convert IPS files to SDB (simple database) format
#
# Tom Moore, September 5, 1995
#
#
# This procedure reads in the IPS datasets and converts the format 
# of the ips.cls, ips.scl, inv.start, ips.crv, ips.ccs
# and ips.mtx files into something that is easier to deal with in
# the HSG/TCL system;
#

puts "Converting IPS file formats"

#
# Class File
#
puts "Class file (class.lu)..."
set in [open "ips.cls" r]
set out [open "class.lu" w]
puts $out "VAR InitialClass TYPE InitialCurveset SITE : AGE ClassAREA RESERVE"
while {[gets $in record] != -1} {
    set class [string trim [string range $record 0 3]]
    set type [string trim [string range $record 4 9]]
    set age [string trim [string range $record 10 12]]
    set area [string trim [string range $record 13 20]]
    set curveset [string trim [string range $record 21 23]]
    set site [string trim [string range $record 24 24]]
    set reserve [string trim [string range $record 25 27]]
    regsub -all "," $area "" area
    puts $out [list DATA $class $type $curveset $site : $age $area $reserve]
}
close $in
close $out

#
# Stand-Class list
#
puts "Stand-class list (stand.lu)..."
set in [open "ips.scl" r]
set out [open "stand.lu" w]
puts $out "VAR STAND InitialClass :"
while {[gets $in record] != -1} {
    puts $out [list DATA [string trim [string range $record 0 7]] [string trim [string range $record 8 11]] :]
}
close $in
close $out

#
# Starting inventory file
#
puts "Starting inventory (stand.inv)..."

set in [open "inv.start" r]
set out [open "stand.inv" w]
puts $out "VAR STAND InitialMaturity CrownClosure InitialFuna Owner : Area"
while {[gets $in record] != -1} {
    set stand [string trim [string range $record 0 8]]
    set area [expr [string range $record 9 21] / 10000]
    set maturity [string trim [string range $record 22 23]]
    set crown [string trim [string range $record 24 26]]
    set funa [string trim [string range $record 27 32]]
    set owner [string trim [string range $record 33 35]]
    puts $out [list DATA $stand $maturity $crown $funa $owner : $area]
}
close $in 
close $out

#
# Curve set 
#
puts "Curve set (curve.yld, set.lu)..."

#
# First scan the file to determine the valid list of codes
#
if {[info exist compArray]} {unset compArray}
set in [open "ips.crv" r]
while {[gets $in record] != -1} {
    set type [string trim [string range $record 9 9]]
    if {$type != "V"} continue
    set code [string trim [string range $record 19 24]]
    set compArray([set code]) $code
}
set components {}
foreach code [lsort [array names compArray]] {
    regsub -all " " $code "_" code
    regsub -all "/" $code "_" code
    lappend components $code
}
close $in

#
# now go through the curve file again, this time convert the format
#
set in [open "ips.crv" r]
set out1 [open "curve.yld" w]
set out2 [open "set.lu" w]
puts $out2 "VAR Curveset ClassName BreakupCurve CompList : EntryAge BreakupAge"
puts $out1 "VAR Curveset : Age $components"

set oldCurveset {}
set compList {}
if {[info exists volList]} {unset volList}

while {[gets $in record] != -1} {
    set type [string trim [string range $record 9 9]]
    if {$type != "V"} continue

    set curveset [string trim [string range $record 0 2]]
    if {$curveset != $oldCurveset && [llength $compList] > 0} {
	puts $out2 [list DATA $oldCurveset $class $breakup $compList : $entryAge $breakupAge]
	set compList {}
	for {set i 0} {$i < $nvert} {incr i} {
	    set age [expr ($i+1)*10]
	    set elements {}
	    foreach comp $components {
		if {[info exists volList([list $comp $age])]} \
			{lappend elements [set volList([list $comp $age])]} \
			{lappend elements 0}
	    }
	    puts $out1 "DATA $oldCurveset : $age $elements"
	}
    }
    set oldCurveset $curveset
    set class [string trim [string range $record 3 8]]
    set breakup [string trim [string range $record 10 12]]
    set entryAge [string trim [string range $record 13 15]]
    set breakupAge [string trim [string range $record 16 18]]
    set code [string trim [string range $record 19 24]]
    regsub -all " " $code "_" code
    regsub -all "/" $code "_" code
    set scale [expr [string range $record 25 27] / 100.]
    set nvert [string range $record 28 30]
    lappend compList $code
    set next 31
    for {set i 0} {$i < $nvert} {incr i} {
	set chunk [string range $record $next [expr $next + 6]]

	set volList([list $code [string trim [string range $chunk 0 2]]]) [format "%-.17g" [expr [string range $chunk 3 5] * $scale]]
	incr next 6
    }
}    
close $in
close $out1
close $out2

#
# Impact matrix, yield curve format
#
puts "Impact curve (matrix.yld)..."
proc put_matrix {hardwood maturity severity bf sp} {
    global out
    puts $out "DATA $maturity $severity $hardwood : 0 1 1"
    puts $out "DATA $maturity $severity $hardwood : 5 [lindex $bf 0] [lindex $sp 0]"
    puts $out "DATA $maturity $severity $hardwood : 10 [lindex $bf 1] [lindex $sp 1]"
    puts $out "DATA $maturity $severity $hardwood : 15 [lindex $bf 2] [lindex $sp 2]"
    puts $out "DATA $maturity $severity $hardwood : 20 [lindex $bf 3] [lindex $sp 3]"
    puts $out "DATA $maturity $severity $hardwood : 25 [lindex $bf 4] [lindex $sp 4]"
    puts $out "DATA $maturity $severity $hardwood : 30 [lindex $bf 5] [lindex $sp 5]"
}
set in [open "ips.mtx" r]
set out [open "matrix.yld" w]
puts $out "VAR Maturity Severity HdwdCode : YrSinceOB BF_Impact SP_Impact"
#
# Add some wildcard records that have no impact
#
puts $out "DATA * * * : 0 1 1"
puts $out "DATA * * * : 9999 1 1"
foreach maturity {I M} {
    foreach severity {Moderate Severe} {
	set BF [gets $in]
	set BFHW [gets $in]
	set SP [gets $in]
	set SPHW [gets $in]
	put_matrix "none" $maturity $severity $BF $SP
	put_matrix "some" $maturity $severity $BFHW $SPHW
    }
}
close $in
close $out

#
# Schedule file
#
puts "Harvest schedule file (schedule.dat)..."
set in [open "ips.ccs" r]
set out [open "schedule.dat" w]
puts $out "VAR STAND PERIOD :"
while {[gets $in record] != -1} {
    puts $out [list DATA [string trim [string range $record 0 7]] [string trim [string range $record 8 end]] :]
}    
close $in
close $out

puts "Done"


