proc sdb2info {sdbdb infofile} {

    #
    # Procedure to copy an SDB file to an INFO database file.
    # The INFO file will be created and items defined automatically
    #
    # Arguments:
    #    sdbdb    --   The name of the sdb cursor
    #    infofile --   The name of the info database file (e.g.  "a@tstdr9")
    #
    # Side effects:
    #  Creates an info file named $infofile
    #
    # Returns:
    #  A count of the number of records copied
    #

    if {[lsearch -exact [info commands] $sdbdb] == -1} {
	error "sdb cursor $sdbdb does not exist"
    }

    for {set i 0} \
	    {[lsearch -exact [info commands] [format "A%d" $i]] != -1} \
	    { incr i } \
	    {}
    set infodb [format "A%d" $i]

    if {[infolib exist $infofile]} {
	puts "deleting the existing database file"
	infolib delete $infofile
    }

    #
    # first build up an item list.  This will be used to create
    # the new file.
    #
    set item_list {}
    foreach item [$sdbdb output] {
	if {[string compare [lindex [$sdbdb info item $item] 1] "CODE"] == 0} {
	    set lmax 0
	    foreach code [$sdbdb info codes $item] {
		if {[string length $code] > $lmax} {
		    set lmax [string length $code]
		}
	    }
	    lappend item_list [list $item c $lmax 0]
	} else {
	    lappend item_list [list $item f 8 3]
	}
    }

    #
    # Create the info file
    #
    infolib create $infodb $infofile $item_list

    #
    # Loop through and transfer the records
    #
    for {set pos 0} {$pos < [$sdbdb info num_records]} {incr pos} {
	$infodb put next [$sdbdb get $pos]
    }

    #
    # close the file and return
    #
    set count [$infodb info num_records]
    $infodb close
    return $count
}
	
proc info2sdb {infodb sdbdb} {

    #
    # Procedure to copy an INFO database to an SDB database.
    # The SDB database will be created and items defined automatically
    #
    # Arguments:
    #    infodb   --   The name of the existing info cursor
    #    sdbdb    --   The name of the sdb cursor to be created
    #
    # Side effects:
    #   Opens up an SDB cursor named $sdbdb
    #
    # Returns:
    #   The number of records copied.
    #

    #
    # Check arguments
    #
    if {[lsearch -exact [info commands] $infodb] == -1} {
	error "infolib cursor $infodb does not exist"
    }

    if {[lsearch -exact [info commands] $sdbdb] != -1} {
	error "sdb cursor $sdbdb already exists"
    }

    #
    # create the empty SDB database
    #
    sdb create $sdbdb [$infodb info num_records]

    #
    # create the fields in the SDB database based on the fields
    # in the INFO database
    #
    set item_list {}
    foreach item [$infodb info items] {
	if {[lsearch -exact { C D } [string toupper [lindex [$infodb info item $item] 1]] ] != -1} {
	    $sdbdb allocate $item CODE
	} else {
	    $sdbdb allocate $item NUMBER
	}
    }

    #
    # Copy the records
    #
    for {set pos 0} {$pos < [$sdbdb info num_records]} {incr pos} {
	$sdbdb put $pos [$infodb get [expr $pos + 1]]
    }

    #
    # Return the count of the size of the database
    #
    return [$sdbdb info num_records]
}
   